#==============================================================================
# Title: County Guessing Game
# Author: Ryan Slater
# Date: 7/28/2017
# Purpose: Game where player guesses as many countries as they can
#==============================================================================

def CountryGuessingGame():
    countryList = ['afghanistan', 'albania', 'algeria', 'andorra', 'angola', 'antigua and barbuda', 'argentina', 'armenia', 'aruba', 'australia', 'austria', 'azerbaijan', 'bahamas', 'bahrain', 'bangladesh', 'barbados', 'belarus', 'belgium', 'belize', 'benin', 'bhutan', 'bolivia', 'bosnia and herzegovina', 'botswana', 'brazil', 'brunei', 'bulgaria', 'burkina faso', 'burma', 'burundi', 'cambodia', 'cameroon', 'canada', 'cabo verde', 'central africian republic', 'chad', 'chile', 'china', 'colombia', 'comoros', 'democratic republic of the congo', 'republic of the congo', 'costa rica', 'cote d\'ivoire', 'croatia', 'cuba', 'curacao', 'cyprus', 'czechia', 'denmark', 'djibouti', 'dominica', 'dominican republic', 'ecuador', 'egypt', 'el salvador', 'equatorial guinea', 'eritrea', 'estonia', 'ethiopia', 'fiji', 'finland', 'france', 'gabon', 'gambia', 'georgia', 'germany', 'ghana', 'greece', 'grenada', 'guatemala', 'guinea', 'guinea-bissau', 'huyana', 'haiti', 'holy see', 'honduras', 'hong kong', 'hungary', 'iceland', 'india', 'indonesia', 'iran', 'iraq', 'ireland', 'israel', 'italy', 'jamaica', 'japan', 'jordan', 'kazakhstan', 'kenya', 'kiribati', 'north korea', 'south korea', 'kosovo', 'kuwait', 'kyrgyzstan', 'laos', 'latvia', 'lebanon', 'lesotho', 'liberia', 'libya', 'liechtenstein', 'lithuania', 'luxembourg', 'macau', 'macedonia', 'madagascar', 'malawi', 'malasia', 'maldives', 'mali', 'malta', 'marshall islands', 'mauritania', 'mauritius', 'mexico', 'micronesia', 'moldova', 'monaco', 'mongolia', 'montenegro', 'morocco', 'mozambique', 'namibia', 'nauru', 'nepal', 'netherlands', 'new zealand', 'nicaragua', 'niger', 'nigeria', 'norway', 'oman', 'pakistan', 'palau', 'palestinian territories', 'panama', 'papau new guinea', 'paraguay', 'peru', 'philippines', 'poland', 'portugal', 'qatar', 'romania', 'russia', 'rwanda', 'saint kitts and nevis', 'saint lucia', 'saint vincent and the grenadines', 'samoa', 'san marino', 'sao tome and principe', 'saudi arabia', 'senegal', 'serbia', 'seychelles', 'sierra leone', 'singapore', 'sint maarten', 'slovakia', 'slovenia', 'solomon islands', 'somalia', 'south africa', 'spain', 'sri lanka', 'sudan', 'south sudan', 'suriname', 'swaziland', 'sweden', 'switzerland', 'syria', 'taiwan', 'tajikistan', 'tanzania', 'thailand', 'timor-leste', 'togo', 'tonga', 'trinidad and tobago', 'tunisia', 'turkey', 'turkmenistan', 'tuvalu', 'uganda', 'ukraine', 'united arab emirates', 'united kingdom', 'united states of america', 'uruguay', 'uzbekistan', 'vanuatu', 'venezuela', 'vietnam', 'yemen', 'zambia', 'zimbabwe']

    correctGuesses  = []
    turn = 1
    userQuits = False
    while turn <= len(countryList) and userQuits == False:
        print(100*'\n')
        for i in range(len(correctGuesses)):
            if i == len(correctGuesses)-1:
                print(correctGuesses[i])
            else:
                print(correctGuesses[i] + ', ', end='')
        print(str(len(correctGuesses)) + '/' + str(len(countryList)))
        print('Guess a country or enter "quit" to give up:')
        guess = input()
        if guess.lower() == 'quit':
            userQuits == True
            break
        if guess.lower() in countryList:
            correctGuesses.append(guess)
            print('That\'s right!')
        else:
            print('That\'s not a country!')

    if len(correctGuesses) == len(countryList):
        print('Congratulations, you guessed all ' + str(len(countryList)) + ' countries correctly!')
    else:
        print('You guessed ' + str(len(correctGuesses)) + '/' + str(len(countryList)) + ' countries correctly!')



# =============================================================================
# Battleship AI generate shots
# =============================================================================

import matplotlib.pyplot as plt
import random as rand
import numpy as np

AIBoard = np.array([[' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '],
                    [' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ']])

def updateMode():
    """
    Updates the AI mode

    Returns
    -------------------

    mode: str
        "target" or "search"
    """
    for row in range(0, 10):
        for col in range(0, 10):
            if AIBoard[row][col] == 'X':
                return "target"
    return "search"

def updateAIBoardSinking(shipCoords):
    """
    Call once a ship is sunk. Replaces the 'X's on the AI board with 'S's to denote a sunk ship
    """
    for i in range(0, len(shipCoords)):
        row = shipCoords[i][0]
        col = shipCoords[i][1]
        AIBoard[row][col] = 'S'

def updateAIBoard(guess, value):
    """
    Mutator function for AIBoard

    Parameters
    -------------------

    guess: Tuple (int, int)
        Coordinates of guess (row, column)
    value: str
        Use 'O' or 'X'
    """
    AIBoard[guess[0]][guess[1]] = value

def checkValidShotLocation(guess):
    """
    Checks if a shot location is valid

    Parameters
    -------------------

    guess: Tuple (int, int)
        Coordinates of guess (row, column)

    Returns
    -------------------

    Boolean, true if valid location, false if not
    """
    if str(guess[0]) in "0123456789" and str(guess[1]) in "0123456789":
        if AIBoard[guess[0]][guess[1]] == 'O' or AIBoard[guess[0]][guess[1]] == 'X' or AIBoard[guess[0]][guess[1]] == 'S':
            return False
        return True
    return False

def __saveHeatMap__(ships, mode, diff, turn, directory, name):
    if mode == "search":
        search(diff, ships, AIBoard, turn)
    else:
        target(ships)
    a = AIBoardHeatMap
    plt.imshow(a, cmap='binary')
    plt.title(name + ' ' + str(turn))
    plt.savefig(directory + '/' + str(turn) + '.png')

def search(diff, ships, p2board, turnCount):
    global AIBoardHeatMap
    AIBoardHeatMap = np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])

    method = 3
    randomProb = 25
    if turnCount > 50:
        randomProb = 0
    if rand.randint(1, 100) <= randomProb:
        #random
        method = 1
        if rand.randint(1, 100) <= 65:
            #noisemap
            method = 0
    elif rand.randint(1, 100) <= 10:
        #corner
        method = 2

    #NOISEMAP
    if method == 0:
        shipCount = 0
        if ships[0] == False:
            shipCount += 1
        if ships[1] == False:
            shipCount += 1
        if ships[2] == False:
            shipCount += 1
        if ships[3] == False:
            shipCount += 1
        if ships[4] == False:
            shipCount += 1
        for row in range(0, 10):
            for col in range(0, 10):
                AIBoardHeatMap[row][col] = rand.randint(0, shipCount)
        method = 3

    #RANDOM SHOT
    if method == 1:
        while True:
            shot = (rand.randint(0, 9), rand.randint(0, 9))
            if AIBoard[shot[0]][shot[1]] == ' ':
                return shot

    #RANDOM CORNER
    if method == 2:
        c0, c1, c2, c3 = False, False, False, False
        if AIBoard[0][0] == ' ':
            c0 = True
        if AIBoard[0][9] == ' ':
            c1 = True
        if AIBoard[9][0] == ' ':
            c2 = True
        if AIBoard[9][9] == ' ':
            c3 = True

        shot = (0, 0)
        if c0 or c1 or c2 or c3:
            while True:
                corner = rand.randint(0, 3)
                if corner == 0:
                    if c0 == True:
                        shot = (0, 0)
                        break
                elif corner == 1:
                    if c1 == True:
                        shot = (0, 9)
                        break
                elif corner == 2:
                    if c2 == True:
                        shot = (9, 0)
                        break
                elif corner == 3:
                    if c3 == True:
                        shot = (9, 9)
                        break
            return shot

        else:
            method = 3

    #HEATMAP
    if method == 3:
        #For each space
        for row in range(0, 10):
            for col in range(0, 10):

                #Destroyer Check
                if ships[4] == False and p2board[row][col] == ' ':
                    if col-1 >= 0:
                        if AIBoard[row][col-1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-1 >= 0:
                        if AIBoard[row-1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col+1 <= 9:
                        if AIBoard[row][col+1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row+1 <= 9:
                        if AIBoard[row+1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1

                #Sub Check
                if ships[3] == False and p2board[row][col] == ' ' and diff >= 2:
                    if col-1 >= 0 and col+1 <= 9:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col+1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-1 >= 0 and row+1 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row+1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col-2 >= 0:
                        if AIBoard[row][col-2] == ' ' and AIBoard[row][col-1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-2 >= 0:
                        if AIBoard[row-2][col] == ' ' and AIBoard[row-1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col+2 <= 9:
                        if AIBoard[row][col+2] == ' ' and AIBoard[row][col+1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row+2 <= 9:
                        if AIBoard[row+2][col] == ' ' and AIBoard[row+1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1

                #Cruiser Check
                if ships[2] == False and p2board[row][col] == ' ' and diff >= 2:
                    if col-1 >= 0 and col+1 <= 9:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col+1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-1 >= 0 and row+1 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row+1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col-2 >= 0:
                        if AIBoard[row][col-2] == ' ' and AIBoard[row][col-1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-2 >= 0:
                        if AIBoard[row-2][col] == ' ' and AIBoard[row-1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col+2 <= 9:
                        if AIBoard[row][col+2] == ' ' and AIBoard[row][col+1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row+2 <= 9:
                        if AIBoard[row+2][col] == ' ' and AIBoard[row+1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1

                #Battleship Check
                if ships[1] == False and p2board[row][col] == ' ' and diff == 3:
                    if col-3 >= 0:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col-2] == ' ' and AIBoard[row][col-3] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col-2 >= 0 and col+1 <= 9:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col-2] == ' ' and AIBoard[row][col+1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col-1 >= 0 and col+2 <= 9:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col+1] == ' ' and AIBoard[row][col+2] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col+3 <= 9:
                        if AIBoard[row][col+1] == ' ' and AIBoard[row][col+2] == ' ' and AIBoard[row][col+3] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-3 >= 0:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row-2][col] == ' ' and AIBoard[row-3][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-2 >= 0 and row+1 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row-2][col] == ' ' and AIBoard[row+1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-1 >= 0 and row+2 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row+1][col] == ' ' and AIBoard[row+2][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row+3 <= 9:
                        if AIBoard[row+1][col] == ' ' and AIBoard[row+2][col] == ' ' and AIBoard[row+3][col] == ' ':
                            AIBoardHeatMap[row][col] += 1

                #Carrier Check
                if ships[0] == False and p2board[row][col] == ' ' and diff == 3:
                    if col-4 >= 0:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col-2] == ' ' and AIBoard[row][col-3] == ' ' and AIBoard[row][col-4] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col-3 >= 0 and col+1 <= 9:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col-2] == ' ' and AIBoard[row][col-3] == ' ' and AIBoard[row][col+1] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col-2 >= 0 and col+2 <= 9:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col-2] == ' ' and AIBoard[row][col+1] == ' ' and AIBoard[row][col+2] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col-1 >= 0 and col+3 <= 9:
                        if AIBoard[row][col-1] == ' ' and AIBoard[row][col+1] == ' ' and AIBoard[row][col+2] == ' ' and AIBoard[row][col+3] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if col+4 <= 9:
                        if AIBoard[row][col+1] == ' ' and AIBoard[row][col+2] == ' ' and AIBoard[row][col+3] == ' ' and AIBoard[row][col+4] == ' ':
                            AIBoardHeatMap[row][col] += 1

                    if row-4 >= 0:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row-2][col] == ' ' and AIBoard[row-3][col] == ' ' and AIBoard[row-4][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-3 >= 0 and row+1 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row-2][col] == ' ' and AIBoard[row-3][col] == ' ' and AIBoard[row-1][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-2 >= 0 and row+2 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row-2][col] == ' ' and AIBoard[row+1][col] == ' ' and AIBoard[row+2][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row-1 >= 0 and row+3 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row+1][col] == ' ' and AIBoard[row+2][col] == ' ' and AIBoard[row+3][col] == ' ':
                            AIBoardHeatMap[row][col] += 1
                    if row+4 <= 9:
                        if AIBoard[row-1][col] == ' ' and AIBoard[row+2][col] == ' ' and AIBoard[row+3][col] == ' ' and AIBoard[row+4][col] == ' ':
                            AIBoardHeatMap[row][col] += 1

        maxVal = 0
        target = []
        for row in range(0, 10):
            for col in range(0, 10):
                if AIBoardHeatMap[row][col] > maxVal:
                    maxVal = AIBoardHeatMap[row][col]

        while True:
            target = (rand.randint(0, 9), rand.randint(0, 9))
            if AIBoardHeatMap[target[0]][target[1]] == maxVal:
                return target









def target(ships):
    global AIBoardHeatMap
    AIBoardHeatMap = np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                               [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])

    #For each space
    for row in range(0, 10):
        for col in range(0, 10):

            if AIBoard[row][col] == 'X':
                if col-1 >= 0 and AIBoard[row][col-1] == ' ':
                    AIBoardHeatMap[row][col-1] += 1
                if col+1 <= 9 and AIBoard[row][col+1] == ' ':
                    AIBoardHeatMap[row][col+1] += 1
                if row-1 >= 0 and AIBoard[row-1][col] == ' ':
                    AIBoardHeatMap[row-1][col] += 1
                if row+1 <= 9 and AIBoard[row+1][col] == ' ':
                    AIBoardHeatMap[row+1][col] += 1

                if col-1 >= 0 and col+1 <= 9:
                    if AIBoard[row][col+1] == ' ':
                        AIBoardHeatMap[row][col+1] += 1
                    if AIBoard[row][col-1] == 'X':
                        AIBoardHeatMap[row][col+1] += 1
                    if AIBoard[row][col+1] == 'O' or AIBoard[row][col+1] == 'X':
                        AIBoardHeatMap[row][col+1] = 0

                if row-1 >= 0 and row+1 <= 9:
                    if AIBoard[row+1][col] == ' ':
                        AIBoardHeatMap[row+1][col] += 1
                    if AIBoard[row-1][col] == 'X':
                        AIBoardHeatMap[row+1][col] += 1
                    if AIBoard[row+1][col] == 'O' or AIBoard[row+1][col] == 'X':
                        AIBoardHeatMap[row+1][col] = 0

                if col+1 <= 9 and col-1 >= 0:
                    if AIBoard[row][col-1] == ' ':
                        AIBoardHeatMap[row][col-1] += 1
                    if AIBoard[row][col+1] == 'X':
                        AIBoardHeatMap[row][col-1] += 1
                    if AIBoard[row][col-1] == 'O' or AIBoard[row][col-1] == 'X':
                        AIBoardHeatMap[row][col-1] = 0

                if row+1 <= 9 and row-1 >= 0:
                    if AIBoard[row-1][col] == ' ':
                        AIBoardHeatMap[row-1][col] += 1
                    if AIBoard[row+1][col] == 'X':
                        AIBoardHeatMap[row-1][col] += 1
                    if AIBoard[row-1][col] == 'O' or AIBoard[row-1][col] == 'X':
                        AIBoardHeatMap[row-1][col] = 0

    maxVal = 0
    for row in range(0, 10):
        for col in range(0, 10):
            if AIBoardHeatMap[row][col] > maxVal:
                maxVal = AIBoardHeatMap[row][col]

    for row in range(0, 10):
        for col in range(0, 10):
            if AIBoardHeatMap[row][col] == maxVal:
                return (row, col)



"""RyGames
Ryan J. Slater
"""

__title__ = 'rygames'
# version may have no more then numerical digits after decimal point.
# 1.11 is actually a higher release than 1.2 (confusing)
__version__ = '0.1a12'
__author__ = u'Ryan J. Slater'
__license__ = ''
__copyright__ = 'Copyright 2017 Ryan J. Slater'
"""__all__ = ['sdof', 'mdof', 'ema', 'vibesystem', 'continuous_systems',
           '__version__']"""

"""
If the __all__ above is commented out, this code will then execute to
completion, as the default behaviour of import * is to import all symbols
that do not begin with an underscore, from the given namespace.

Reference:
https://docs.python.org/3.5/tutorial/modules.html#importing-from-a-package
"""

import pygame
import random as rand
import numpy as np
import matplotlib.pyplot as plt
from PyDictionary import PyDictionary
from nltk.corpus import words
import names
import os
import time
from rygames import __AIFleets__
from rygames import __AIShots__
from rygames.CoinGame import CoinGame
from rygames.CountryGuessingGame import CountryGuessingGame
from rygames.Hangman import Hangman
from rygames.TicTacToe1Player import TicTacToe1Player
from rygames.TicTacToe2Player import TicTacToe2Player
from rygames.TwentyFortyEight import TwentyFortyEight
from rygames.Warships import Warships



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#==============================================================================
# Title: 2048
# Author: Ryan J Slater
# Date: 12/25/2017
#==============================================================================

import pygame
import random as rand
import numpy as np

class __colors__():
    BLACK = (0,0,0)
    WHITE = (255,255,255)
    RED = (200,0,0)
    GREEN = (0,200,0)
    BLUE = (0, 0, 200)
    BRIGHTRED = (255,0,0)
    BRIGHTGREEN = (0,255,0)
    BRIGHTBLUE = (0, 0, 255)
    BOARD = (187, 173, 160)

    def __getTileColor__(number):
        if number == 0:
            return((248, 238, 228,))
        elif number == 2:
            return((238, 228, 218))
        elif number == 4:
            return((237, 224, 200))
        elif number == 8:
            return((242, 177, 121))
        elif number == 16:
            return((245, 149, 99))
        elif number == 32:
            return((246, 124, 95))
        elif number == 64:
            return((246, 94, 59))
        elif number == 128:
            return((237, 207, 114))
        elif number == 256:
            return((237, 204, 97))
        elif number == 512:
            return((237, 200, 80))
        elif number == 1024:
            return((237, 197, 63))
        elif number == 2048:
            return((237, 194, 46))
        else:
            if number >= 131072:
                return((0, 0, 0))
            val = 4096
            color = 50
            while True:
                if number == val:
                    return((color, color, color))
                else:
                    color -= 10
                    val *= 2

    def __getTileTextColor__(number):
        if number >= 8:
            return((249, 246, 242))
        return((119, 110, 101))

def __text_objects__(text, font):
    textSurface = font.render(text, True, __colors__.BLACK)
    return textSurface, textSurface.get_rect()

def __tileText__(text, font, val):
    textSurface = font.render(text, True, __colors__.__getTileTextColor__(val))
    return textSurface, textSurface.get_rect()

def __button__(msg, x, y, w, h, ic, ac, action=None):
    mouse = pygame.mouse.get_pos()
    click = pygame.mouse.get_pressed()
    if x+w > mouse[0] > x and y+h > mouse[1] > y:
        pygame.draw.rect(gameDisplay, ac, (x, y, w, h))
        if click[0] == 1 and action != None:
            if action == play:
                play(displayWidth, displayHeight)
            elif action == __optionMenu__:
                __optionMenu__()
            else:
                action()
    else:
        pygame.draw.rect(gameDisplay, ic, (x, y, w, h))
    text = pygame.font.SysFont('comicsansms', __button__TextSize)
    textSurf, textRect = __text_objects__(msg, text)
    textRect.center = ((x+(w/2)), (y+(h/2)))
    gameDisplay.blit(textSurf, textRect)

def __unpause__():
    global pause
    pause = False

def __paused__():
    text = pygame.font.SysFont('comicsansms', __titleTextSize__)
    TextSurf, TextRect = __text_objects__('Paused', text)
    TextRect.center = ((displayWidth/2), (displayHeight/2))
    gameDisplay.blit(TextSurf, TextRect)
    while pause:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                __quitGame__()
        __button__('CONTINUE', int(displayWidth/2-(125/640)*displayWidth), int((3/4)*displayHeight), int((12/64)*displayWidth), int((5/48)*displayHeight), __colors__.GREEN, __colors__.BRIGHTGREEN, __unpause__)
        __button__('MAIN MENU', int((displayWidth/2)+(5/640)*displayWidth), int((3/4)*displayHeight), int((13/64)*displayWidth), int((5/48)*displayHeight), __colors__.RED, __colors__.BRIGHTRED, play)
        pygame.display.update()
        clock.tick(60)

def __quitGame__():
    pygame.quit()


#==================================================================================================================


def __gameIntro__():
    gameDisplay.fill(__colors__.WHITE)
    largeText = pygame.font.SysFont('comicsansms', __titleTextSize__)
    smallText = pygame.font.SysFont('comicsansms', __button__TextSize)
    TextSurf, TextRect = __text_objects__('2048', largeText)
    TextRect.center = (int(displayWidth/2), int(displayHeight/2))
    gameDisplay.blit(TextSurf, TextRect)
    TextSurf, TextRect = __text_objects__('Ryan J Slater', smallText)
    TextRect.center = ((displayWidth/2), (displayHeight/2)+int((4/48)*displayHeight))
    gameDisplay.blit(TextSurf, TextRect)
    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                __quitGame__()
        __button__('PLAY', int((displayWidth/2)-(165/640)*displayWidth), int((3/4)*displayHeight), int((10/64)*displayWidth), int((5/48)*displayHeight), __colors__.GREEN, __colors__.BRIGHTGREEN, __gameLoop__)
        __button__('OPTIONS', int((displayWidth/2)-(55/640)*displayWidth), int((3/4)*displayHeight), int((11/64)*displayWidth), int((5/48)*displayHeight), __colors__.BLUE, __colors__.BRIGHTBLUE, __optionMenu__)
        __button__('QUIT', int((displayWidth/2)+(65/640)*displayWidth), int((3/4)*displayHeight), int((10/64)*displayWidth), int((5/48)*displayHeight), __colors__.RED, __colors__.BRIGHTRED, __quitGame__)

        pygame.display.update()
        clock.tick(60)

def __changeRes1__():
    global displayWidth
    global displayHeight
    global __button__TextSize
    global __titleTextSize__
    __titleTextSize__ = 77
    __button__TextSize = 20
    displayWidth = 480
    displayHeight = 320
    play(displayWidth, displayHeight)

def __changeRes2__():
    global displayWidth
    global displayHeight
    global __button__TextSize
    global __titleTextSize__
    __titleTextSize__ = 115
    __button__TextSize = 30
    displayWidth = 640
    displayHeight = 480
    play(displayWidth, displayHeight)

def __changeRes3__():
    global displayWidth
    global displayHeight
    global __button__TextSize
    global __titleTextSize__
    __titleTextSize__ = 144
    __button__TextSize = 38
    displayWidth = 1024
    displayHeight = 600
    play(displayWidth, displayHeight)

def __changeRes4__():
    global displayWidth
    global displayHeight
    global __button__TextSize
    global __titleTextSize__
    __titleTextSize__ = 216
    __button__TextSize = 56
    displayWidth = 1200
    displayHeight = 900
    play(displayWidth, displayHeight)

def __changeRes5__():
    global displayWidth
    global displayHeight
    global __button__TextSize
    global __titleTextSize__
    __titleTextSize__ = 259
    __button__TextSize = 68
    displayWidth = 1920
    displayHeight = 1080
    play(displayWidth, displayHeight)

def __optionMenu__():
    # Graphics Options:
        # Color Scheme
        # Display Resolution
    global displayWidth
    global displayHeight
    gameDisplay.fill(__colors__.WHITE)
    text = pygame.font.SysFont('comicsansms', __titleTextSize__)
    TextSurf, TextRect = __text_objects__('Options', text)
    TextRect.center = ((displayWidth/2), int((5/48)*displayHeight))
    gameDisplay.blit(TextSurf, TextRect)

    text = pygame.font.SysFont('comicsansms', __button__TextSize)
    TextSurf, TextRect = __text_objects__('Resolution', text)
    TextRect.center = (int(displayWidth/3), int((11/48)*displayHeight))
    gameDisplay.blit(TextSurf, TextRect)
    TextSurf, TextRect = __text_objects__('Color Scheme', text)
    TextRect.center = (int(2*displayWidth/3), int((11/48)*displayHeight))
    gameDisplay.blit(TextSurf, TextRect)

    text = pygame.font.SysFont('comicsansms', int(__button__TextSize/2))
    TextSurf, TextRect = __text_objects__(str(displayWidth) + 'x' + str(displayHeight), text)
    TextRect.center = (int(displayWidth/3), int((125/480)*displayHeight))
    gameDisplay.blit(TextSurf, TextRect)

    __button__Width = int((11/48)*displayWidth)
    __button__Height =  int((5/48)*displayHeight)

    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                __quitGame__()
        __button__('BACK', int((1/48)*displayWidth), int((1/48)*displayHeight), __button__Width, __button__Height, __colors__.RED, __colors__.BRIGHTRED, play)
        __button__('480x320', int((displayWidth/3)-(__button__Width/2)), int((135/480)*displayHeight), __button__Width, __button__Height, __colors__.BLUE, __colors__.BRIGHTBLUE, __changeRes1__)
        __button__('640x480', int((displayWidth/3)-(__button__Width/2)), int((195/480)*displayHeight), __button__Width, __button__Height, __colors__.BLUE, __colors__.BRIGHTBLUE, __changeRes2__)
        __button__('1024x600', int((displayWidth/3)-(__button__Width/2)), int((255/480)*displayHeight), __button__Width, __button__Height, __colors__.BLUE, __colors__.BRIGHTBLUE, __changeRes3__)
        __button__('1200x900', int((displayWidth/3)-(__button__Width/2)), int((315/480)*displayHeight), __button__Width, __button__Height, __colors__.BLUE, __colors__.BRIGHTBLUE, __changeRes4__)
        __button__('1920x1080', int((displayWidth/3)-(__button__Width/2)), int((375/480)*displayHeight), __button__Width, __button__Height, __colors__.BLUE, __colors__.BRIGHTBLUE, __changeRes5__)
        pygame.display.update()
        clock.tick(60)

def __getInitialBoard__():
    board = np.zeros((4, 4), dtype=int)
    coord = (rand.randint(0, 3), rand.randint(0, 3))
    if rand.randint(0, 100) > 10:
        board[coord] = 2
    else:
        board[coord] = 4
    while True:
        coord2 = (rand.randint(0, 3), rand.randint(0, 3))
        if coord2 != coord:
            board[coord2] = 2
            if rand.randint(0, 100) < 10 and board[coord] != 4:
                board[coord2] = 4
            return board

def __checkLoss__(board):
    zeros = 0
    for row in range(4):
        for col in range(4):
            if board[row][col] == 0:
                zeros += 1
            else:
                if row > 0:
                    if board[row][col] == board[row-1][col]:
                        return False
                if row < 3:
                    if board[row][col] == board[row+1][col]:
                        return False
                if col > 0:
                    if board[row][col] == board[row][col-1]:
                        return False
                if col < 3:
                    if board[row][col] == board[row][col+1]:
                        return False
    if zeros > 0:
        return False
    return True

def __drawTile__(row, col, w, h, origin, val):
    text = [pygame.font.SysFont('comicsansms', int((3/2)*__button__TextSize)), pygame.font.SysFont('comicsansms', int((7/6)*__button__TextSize)), pygame.font.SysFont('comicsansms', int((5/6)*__button__TextSize))]
    pygame.draw.rect(gameDisplay, __colors__.__getTileColor__(val), (origin[0]+col*(w+10), origin[1]+row*(h+10), w, h))
    if val < 1000:
        text = text[0]
    elif val < 10000:
        text = text[1]
    elif val < 100000:
        text = text[2]
    if val != 0:
        TextSurf, TextRect = __tileText__(str(val), text, val)
        TextRect.center = ((origin[0]+col*(w+10)+int(w/2), origin[1]+row*(h+10)+int(h/2)))
        gameDisplay.blit(TextSurf, TextRect)

def __drawBoard__(board, dw, dh):
    gameDisplay.fill(__colors__.WHITE)
    tileWidth, tileHeight = int(dh/5), int(dh/5)
    if dw < dh:
        tileWidth, tileHeight = int(dw/5), int(dw/5)

    origin = (int(dw/2)-2*tileHeight-15, int(dh/2)-2*tileWidth-15)
    pygame.draw.rect(gameDisplay, __colors__.BOARD, (origin[0]-10, origin[1]-10, 4*tileWidth+50, 4*tileHeight+50))
    for row in range(4):
        for col in range(4):
            __drawTile__(row, col, tileWidth, tileHeight, origin, board[row][col])

def __gameLoss__():
    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                __quitGame__()
        largeText = pygame.font.SysFont('comicsansms', __titleTextSize__)
        TextSurf, TextRect = __text_objects__('You Lose', largeText)
        TextRect.center = ((displayWidth/2), (displayHeight/2))
        gameDisplay.blit(TextSurf, TextRect)
        gameDisplay.blit(TextSurf, TextRect)
        __button__('PLAY AGAIN', int((displayWidth/2)-(135/640)*displayWidth), int((3/4)*displayHeight), int((13/64)*displayWidth), int((5/48)*displayHeight), __colors__.GREEN, __colors__.BRIGHTGREEN, __gameLoop__)
        __button__('QUIT', int((displayWidth/2)+(5/640)*displayWidth), int((3/4)*displayHeight), int((10/64)*displayWidth), int((5/48)*displayHeight), __colors__.RED, __colors__.BRIGHTRED, __quitGame__)
        pygame.display.update()
        clock.tick(60)

def __moveRow__(row, direction):
    if direction == 'left' or direction == 'up':
        for i in range(len(row)):
            if row[i] != 0:
                if i > 0:
                    moveto = 0
                    for j in range(i-1, -1, -1):
                        if row[j] != 0:
                            moveto = j
                            break
                    if row[moveto] != 0:
                        if row[moveto] != row[i]:
                            moveto += 1
                    if moveto != i:
                        row[moveto] += row[i]
                        row[i] = 0
                else:
                    i += 1
    else:
        for i in range(len(row)-1, -1, -1):
            if row[i] != 0:
                if i < len(row)-1:
                    moveto = len(row)-1
                    for j in range(i+1, len(row)):
                        if row[j] != 0:
                            moveto = j
                            break
                    if row[moveto] != 0:
                        if row[moveto] != row[i]:
                            moveto -= 1
                    if moveto != i:
                        row[moveto] += row[i]
                        row[i] = 0
                else:
                    i -= 1
    return row

def __shift__(board, direction):
    origBoard = np.copy(board)
    if direction == 'left' or direction == 'right':
        for row in range(4):
            tmprow = np.array([0, 0, 0, 0])
            for col in range(4):
                tmprow[col] = board[row][col]
            tmprow = __moveRow__(tmprow, direction)
            for i in range(len(tmprow)):
                board[row][i] = tmprow[i]

    elif direction == 'up' or direction == 'down':
        for col in range(4):
            tmpcol = np.array([0, 0, 0, 0])
            for row in range(4):
                tmpcol[row] = board[row][col]
            tmpcol = __moveRow__(tmpcol, direction)
            for i in range(len(tmpcol)):
                board[i][col] = tmpcol[i]

    if np.array_equal(origBoard, board):
        return(board)

    while True:
        coord = (rand.randint(0, 3), rand.randint(0, 3))
        if board[coord] == 0:
            if rand.randint(0, 100) > 10:
                board[coord] = 2
            else:
                board[coord] = 4
            return(board)

def __gameLoop__():
    global pause
    board = __getInitialBoard__()

    while True:
        if __checkLoss__(board):
            __gameLoss__()
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                __quitGame__()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_LEFT or event.key == pygame.K_a:
                    board = __shift__(board, 'left')
                if event.key == pygame.K_RIGHT or event.key == pygame.K_d:
                    board = __shift__(board, 'right')
                if event.key == pygame.K_DOWN or event.key == pygame.K_s:
                    board = __shift__(board, 'down')
                if event.key == pygame.K_UP or event.key == pygame.K_w:
                    board = __shift__(board, 'up')
                if event.key == pygame.K_ESCAPE or event.key == pygame.K_p:
                    pause = True
                    __paused__()
        __drawBoard__(board, displayWidth, displayHeight)
        pygame.display.update()
        clock.tick(60)

def play(dw=640, dh=480):
    global displayWidth
    global displayHeight
    global pause
    global gameDisplay
    global clock
    global options

    pygame.init()
    displayWidth = dw
    displayHeight = dh
    pause = False
    gameDisplay = pygame.display.set_mode((displayWidth, displayHeight))
    pygame.display.set_caption('2048')
    clock = pygame.time.Clock()

    __gameIntro__()
    __gameLoop__()
    __quitGame__()

def TwentyFortyEight():
    global __titleTextSize__
    global __button__TextSize
    __titleTextSize__ = 115
    __button__TextSize = 30
    play()



#!/usr/bin/env python

#from distutils.core import setup
from setuptools import setup
import os
import sys

if sys.version_info < (3, 6):
    sys.exit('Sorry, Python < 3.6 is not supported.')
# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

with open('rygames/__init__.py', 'rb') as fid:
    for line in fid:
        line = line.decode('utf-8')
        if line.startswith('__version__'):
            version = line.strip().split()[-1][1:-1]
            break

download_url = ('https://github.com/rjslater2000/RyGames/\
                blob/master/dist/rygames-' + version + '.whl')
#https://github.com/rjslater2000/RyGames/archive/0.1a1.zip

setup(name='RyGames',
      version=version,
      description=('Python games by Ryan J. Slater'),
      author=u'Ryan J. Slater',
      author_email='ryan.j.slater.2@gmail.com',
      url='https://github.com/rjslater2000/RyGames',
      packages=['rygames'],
      package_data={'rygames': ['../README.md', 'data/*.mat'],
                        '': ['README.md']},
      long_description=read('README.md'),
      keywords=['games', 'battleship', 'python'],
      install_requires=['pygame', 'numpy', 'matplotlib', 'PyDictionary', 'nltk', 'names'],
      )

# https://docs.python.org/3/distutils/setupscript.html#additional-meta-data
